--[[
**********************************************************************
MagicDKP Database - Stores DKP data imported via eqDKP Plus getdkp.php
**********************************************************************
This file is part of MagicDKP, a World of Warcraft Addon

MagicDKP is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MagicDKP is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with MagicDKP.  If not, see <http://www.gnu.org/licenses/>.
**********************************************************************
]]


local MODULE_NAME = "Database"
local mod = MagicDKP
local module = mod:NewModule(MODULE_NAME)
local db
local debug
local dkpdb
--[===[@debug@
debug = true
--@end-debug@]===]
if debug then
   MDB = module
end
local defaults = {
   clearGlobals = true,
   filterZero = true, 
   filterRank = true,
}
   
function module:OnInitialize()
   dkpdb = MagicDKP_Database or {
      dkp = {},
      multidkp = {},
      lastImportDate = 0
   }
   MagicDKP_Database = dkpdb
   
   db = mod:GetModuleDatabase(MODULE_NAME, defaults, module.options)

   db.dkp = nil
   db.multidkp = nil
   db.lastImportDate = nil
   
   if DKPInfo then
      module:ImportGetDKPData()
   end
   if db.clearGlobals then
      multiTable = nil
      DKPInfo = nil
      gdkp = nil
      DKP_ITEMS = nil
      getdkp_itemids = nil
      gdkp_alliases = nil
      GetDKPRaidPlaner = nil
   end

   module.multiColumns = { }
   if dkpdb.multidkp then
      for key, data in pairs(dkpdb.multidkp) do
	 module.multiColumns[key] = {
	    earned = key.."_earned", 
	    spent = key.."_spend",
	    adjust = key.."_adjust",
	    current = key.."_current", 
	 }
      end
   end
end

function module:GetMultiDKP()
   return module.multiColumns
end

function module:GetPlayerData(player)
   return dkpdb.dkp[player]
end

function module:GetPlayerItems(player)
   local data = dkpdb.dkp[player] and dkpdb.dkp[player].items
   if not data or #data == 0 then return end
   
   table.sort(data, function(a, b) return a.dkp > b.dkp end)
   return data
end


function module:GetPlayerDKP(player, multi)
   local dkp = dkpdb.dkp[player]
   local cols = module.multiColumns[multi or "dkp"]
   if dkp and cols then
      return dkp[cols.current],  dkp[cols.earned],  dkp[cols.spent],  dkp[cols.adjust]
   end
end

function module:GetItemBidHistory(item)
   local result = {}
   local name = GetItemInfo(item)
   if type(item) == "string" then
      item = mod:LinkToItemID(item)
   end
   for player, data in pairs(dkpdb.dkp) do
      if data.items then
	 for _,item in pairs(data.items) do
	    if item.itemid == item or item.name == name  then
	       result[#result+1] = {
		  name = player,
		  dkp = item.dkp,
	       }
	    end
	 end
      end
   end
   table.sort(result, function(a, b) return a.dkp > b.dkp end)
   return result
end


-- Import data from the GetDKP file
function module:ImportGetDKPData()
   local isNew = tonumber(DKPInfo.timestamp) > dkpdb.lastImportDate
   if not isNew then return end
   
   dkpdb.dkp = {}
   for name, data in pairs(gdkp.players) do
      if not db.filterZero or data.DKP > 0 then
	 local p = {}
	 for var, val in pairs(data) do
	    if var ~= "class" and var ~= "rcount" then
	       p[var] = val
	    end
	 end
	 dkpdb.dkp[name] = p
      end
   end
   if DKP_ITEMS then
      for name, data in pairs(DKP_ITEMS)  do
	 local p = dkpdb.dkp[name]
	 if p then
	    p.items = {}
	    for id,item in pairs(data.Items) do
	       p.items[id] = {
		  dkp = item.dkp,
		  name = item.name,
		  itemid = getdkp_itemids and getdkp_itemids[item.name]
	       }
	    end
	 end
      end
   end

   dkpdb.multidkp = {}
   if multiTable then
      for _, data in ipairs(multiTable) do 
	 for key, val in pairs(data) do
	    dkpdb.multidkp[key] = {
	       desc = val.disc,
	       events = val.events
	    }
	 end
      end
   end
   dkpdb.lastImportDate = tonumber(DKPInfo.timestamp)
   module.ImportGetDKPData = nil
end
 
